/*
 * GNOME CPUFreq Applet
 * Copyright (C) 2006 Carlos Garcia Campos <carlosgc@gnome.org>
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Authors : Carlos García Campos <carlosgc@gnome.org>
 */

#ifndef CPUFREQ_UTILS_H
#define CPUFREQ_UTILS_H

#include <glib.h>

G_BEGIN_DECLS

/* Useful global methods */
guint    cpufreq_utils_get_n_cpus            (void);
gboolean cpufreq_utils_selector_is_available (void);
gchar   *cpufreq_utils_get_frequency_label   (guint        freq);
gchar   *cpufreq_utils_get_frequency_unit    (guint        freq);

G_END_DECLS

#endif /* CPUFREQ_UTILS_H */
