/*
 * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#ifndef __TRACKER_APPLET_H__
#define __TRACKER_APPLET_H__

#include <libgnome-panel/gp-applet.h>

G_BEGIN_DECLS

#define TRACKER_TYPE_APPLET (tracker_applet_get_type ())
G_DECLARE_FINAL_TYPE (TrackerApplet, tracker_applet,
                      TRACKER, APPLET, GpApplet)

void tracker_applet_setup_about (GtkAboutDialog *dialog);

G_END_DECLS

#endif /* __TRACKER_APPLET_H__ */
