/*
 * Copyright (C) 2014 Alberts Muktupāvels
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef GSETTINGS_H
#define GSETTINGS_H

#define GNOME_DESKTOP_LOCKDOWN_SCHEMA "org.gnome.desktop.lockdown"
#define GNOME_DESKTOP_WM_PREFERENCES  "org.gnome.desktop.wm.preferences"
#define MINI_COMMANDER_GLOBAL_SCHEMA  "org.gnome.gnome-applets.mini-commander.global"
#define MINI_COMMANDER_SCHEMA         "org.gnome.gnome-applets.mini-commander"

#define KEY_MACRO_PATTERNS "macro-patterns"
#define KEY_MACRO_COMMANDS "macro-commands"

#define KEY_AUDIBLE_BELL "audible-bell"

#define KEY_SHOW_HANDLE          "show-handle"
#define KEY_SHOW_FRAME           "show-frame"
#define KEY_AUTOCOMPLETE_HISTORY "autocomplete-history"
#define KEY_NORMAL_SIZE_X        "normal-size-x"
#define KEY_NORMAL_SIZE_Y        "normal-size-y"
#define KEY_SHOW_DEFAULT_THEME   "show-default-theme"
#define KEY_CMD_LINE_COLOR_FG    "cmd-line-color-fg"
#define KEY_CMD_LINE_COLOR_BG    "cmd-line-color-bg"
#define KEY_HISTORY              "history"

#endif
